/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

 /**
 *\file CTarget.cpp
 *\brief  Target rendering class method development.
 */

#include "CTarget.h"
#include <myUtils.h>
#include <iostream>

#define EXTRA	30

IRenderer::Factory registerFactoryTgt("target", CTarget::createObj);

class TargetException : public std::exception {
public:
        TargetException(const char *msg) throw() : errMessage(msg) { } ;
        ~TargetException(void) throw() { } ;
        const char *what(void) const throw() { return errMessage.c_str(); };
private:
        std::string errMessage;
       	TargetException(void) throw();
};

IRenderer* CTarget::createObj(const void* node){
	return new CTarget((const DOMNode*)node);
}

CTarget::CTarget(const DOMNode* node){
	parser_ = new CXMLParser;

	targetPreset((const DOMNode*) node);
	
	//initialize the black image
	image_ = new unsigned char[w_ * h_];
	//Fill image with 0x00
	for(unsigned int j = 0; j < w_ * h_; j++){
		*(image_ + j) =	0x00;
	}
	
	screen_wnd = (Fl_Double_Window *) 0;
	screen_img = (Fl_Box *) 0;
	
	Fl::visual(FL_RGB);
	{
		screen_wnd = new Fl_Double_Window(w_ + EXTRA , h_ + EXTRA, name_.c_str());
		screen_wnd->box(FL_BORDER_BOX);
		{
			screen_img = new Fl_Box(EXTRA / 2, EXTRA / 2, w_, h_);
			screen_img->box(FL_FLAT_BOX);
		}
		screen_wnd->end();
	}
	
	screen_wnd->show();
	Fl::check();	
} 

CTarget::~CTarget(){
	if(screen_wnd) delete screen_wnd;
	if(screen_img) delete screen_img;
	if(image_) delete image_;
	
	if(tnode_) delete tnode_;
	if(parser_) delete parser_;
}

void CTarget::targetPreset(const DOMNode* node){
	char ret[256];
	
	if(!parser_->FindNodeRootat((DOMNode*)node, (char*)"target_preset", tnode_)){
		throw TargetException("target_preset node isn't found");
	}else{
		if (!parser_->GetAttribute(tnode_, (char*)"target_name", (char*)ret)){
			throw TargetException("target_name attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Target name returned: %s\n", ret);
			#endif
			name_ = ret;		/* Target renderer name*/
			IRenderer::checkName(name_);
		}
		
		if (!parser_->GetAttribute(tnode_, (char*)"target_width", (char*)ret)){
			throw TargetException("target_width attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Target width returned: %s\n", ret);
			#endif
			w_ = atol(ret);		/* Target width (in pixel)*/
		}
		
		if (!parser_->GetAttribute(tnode_, (char*)"target_height", (char*)ret)){
			throw TargetException("target_height attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Target height returned: %s\n", ret);
			#endif
			h_ = atol(ret);		/* Target height (in pixel)*/
		}
	}
}


void CTarget::setType(var_type vType){
	vType_ = vType;
	
	if((vType_ != T_UINT8) &&
	   (vType_ != T_UINT16) &&
	   (vType_ != T_UINT32))
		throw TargetException("Data type don't supported!");   
}

void CTarget::render(void* data, unsigned long size){
	unsigned int x, y;
	
	if(size != 2)
		throw TargetException("Only 2D vector are supported");
	
	switch(vType_){
		case T_UINT8:
		x = (unsigned int) *((unsigned char*)data);
		y = (unsigned int) *(((unsigned char*)data) + 1);
		break;
		
		case T_UINT16:
		x = (unsigned int) *((unsigned short*)data);
		y = (unsigned int) *(((unsigned short*)data) + 1);
		break;
		
		case T_UINT32:
		x = *((unsigned int*)data);
		y = *(((unsigned int*)data) + 1);
		break;
		
		default:
		break;
	}

	unsigned char image_24[w_ * h_ * 3];
	static Fl_RGB_Image*	m_image;
	
	fflush(stdout);

	//Convert from gray8 to rgb24
	my_gray8_to_rgb24(image_, image_24, w_, h_);

	//Fix the target
	my_draw_point(image_24, w_, h_, x, y, MY_RED, 6, 3);
	
	m_image = new Fl_RGB_Image(image_24, w_, h_, 3);
	screen_img->image(m_image);
	screen_img->redraw();
	Fl::check();
	
	delete m_image;
}
